/***********************************************************************
frogd, daemon pour l'utilisation des capteurs meteo de http://www.froggyhome.com

Copyright (C) 2001 (Regis Barbier)
Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon 
les termes de la Licence Publique Gnrale GNU publie par la Free Software 
Foundation (version 2 ou bien toute autre version ultrieure choisie par vous).

Ce programme est distribu car potentiellement utile, mais SANS AUCUNE 
GARANTIE, ni explicite ni implicite, y compris les garanties de 
commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la 
Licence Publique Gnrale GNU pour plus de dtails.

Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme 
temps que ce programme. Si ce n'est pas le cas, crivez  la Free Software 
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
*************************************************************************/


#include "frogd.h"
#include "frogd_comm.h"
#include "frogd_calc.h"

SensorType sensortype[] =   
{
 {SENSOR_PRES_TEMP, "Pression-Temperature"},
 {SENSOR_HR, "hygrometrie"},
};

int nbsensortype = 4;

//---------------------------------------------------------------
// become a daemon
//---------------------------------------------------------------
int daemonize(void) {
   if (fork()!=0) {
      exit(0);
   }
   setsid();
   return 1;   
}

//---------------------------------------------------------------
// log the last value generated
//---------------------------------------------------------------
int LogLastResult(Frog* frog) {
   FILE* file;
   struct tm* date;
   time_t t;

   if ( (file = fopen( frog->logfile, frog->logtype)) == NULL ) {
      perror("SaveLastResult can t open the file");
      return -1;
   }

   time(&t);
   date = localtime(&t);
 
   fprintf( file, "%d/%d/%d:%d/%d:%.1f:%.1f:%.1f\n",  date->tm_mday, date->tm_mon+1, date->tm_year+1900, date->tm_hour, date->tm_min, frog->lasttemp, frog->lastpress, frog->lasthum);
  
   fclose(file);
   return 1;   
}

//---------------------------------------------------------------
// Print the last value generated
//---------------------------------------------------------------
int PrintLastResult(Frog* frog) {
   struct tm* date;
   time_t t;

   time(&t);
   date = localtime(&t);
 
   printf("On est %d/%d/%d:%dh%d:\n",  date->tm_mday, date->tm_mon+1, date->tm_year+1900, date->tm_hour, date->tm_min);
   
   printf("**** L'humidite est : %.1f\n", frog->lasthum);
   printf("**** La temperature est : %.1f\n", frog->lasttemp);
   printf("**** La pression est : %.1f\n", frog->lastpress);   
   printf("---------------------------\n");      
   
   return 1;   
}
//---------------------------------------------------------------
// Initialise the frog structure with the defaults values
// read config file
//---------------------------------------------------------------
int InitFrog(Frog* frog) {
   frog->debug   = FROG_DEBUG;
   frog->verbose = FROG_VERBOSE;
   frog->baud    = FROG_SPEED;   
   frog->stopbit = FROG_STOPBIT;
   frog->altitude = FROG_ALTITUDE;
   frog->period = FROG_PERIOD;
   frog->log = FROG_LOG;
   frog->daemon = FROG_DAEMON;   
   frog->nbretry = FROG_NB_RETRY;      
   strcpy(frog->device, FROG_DEVICE);
   strcpy(frog->logfile, FROG_LOGFILE);
   strcpy(frog->logtype, FROG_LOGTYPE);   
   return 1;   
}

//---------------------------------------------------------------
// Print the version and all the data about the frog 
//---------------------------------------------------------------
int PrintIdentification(Frog* frog) {
   int i, j, k;

   printf("Copyright Regis Barbier frog@redge.net\n");
   printf("For more informations please visit :  http://www.froggyhome.com\n");   
   printf("Frogd version             : %s\n", FROG_PROG_VERSION);
   printf("Frogd device              : %s\n", frog->device);
   printf("Frogd altitude            : %d\n", frog->altitude);
   printf("sample period             : %d\n", frog->period);      
   printf("Firmware Version of frog  : %d\n", frog->firmware);
   printf("Serial Number             : %d\n", frog->serialnumber);
   printf("Number of sensors         : %d\n", frog->nbsensor); 
   for( i=0; i<frog->nbsensor; i++) {
      printf("*sensor number %d\n",i);
      printf("    sensor type         : %x", frog->sensor[i].identification);

//we try to determine the type  of sensor
      for (k=0; k<nbsensortype; k++) {
        if ( sensortype[k].identification == (frog->sensor[i].identification & SENSOR_IDENTIFICATION_MASK) )
            printf (" (%s) \n", sensortype[k].name ) ;
      }
      printf("    sensor physical mes : %x\n", frog->sensor[i].phymesurementsize);
      printf("    sensor data size    : %x\n", frog->sensor[i].datasize);
      printf("    sensor data         :  ");

// Now we print the raw data for the sensor
      for(j=0; j<frog->sensor[i].datasize; j++) {
         printf("%.2x ", frog->sensor[i].data[j]);
      }
      printf("\n");
   }
   return 1;   
}

//---------------------------------------------------------------
// Parse the command line to find additionnal arguments
//---------------------------------------------------------------
char parse_cmdline(int argc,char *argv[], Frog* frog) {
   int errflg=0;
   int c;

   while ((c = getopt(argc, argv, "fdvlp:L:D:a:r:")) != EOF) {
      switch (c) {
         case 'f':
            frog->daemon=FALSE;
            break;

         case 'd':
            frog->debug=TRUE;
            break;
            
         case 'v':
            frog->verbose=TRUE  ;
            break;
            
         case 'l':
            frog->log = TRUE;
            break;

         case 'p':
            frog->period = atoi(optarg);
            if (frog->period < FROG_MIN_PERIOD)
            break;

         case 'r':
            frog->nbretry = atoi(optarg);
            break;

         case 'L':
            frog->log = TRUE;
             if (optarg != NULL) {
               strcpy(frog->logfile,optarg);
             }
            break;

         case 'h':
           strcpy(frog->logtype,optarg);
            break;
            
         case 'D':
            strcpy(frog->device,optarg);
             break;
         
         case 'a':
            frog->altitude = atoi(optarg);
            break;

         case '?':
            errflg++;
         break;
            
      }
        if (errflg) {
           printf(     "utilisation: frog [-v] [-d] [-l] [-f] [-h] [-a altitude] [-L fichier] [-D /dev/ttySx] 
	       [-p periode] [-r fois]\n");
           exit (2);
            }
     }
 
    return 1;
}

//---------------------------------------------------------------
// main function ...
//---------------------------------------------------------------
int main(int argc,char *argv[]) {
   Frog frog1;
   Frog* frog;
   int error=FALSE;
   int nberror=0;
   frog=&frog1; 
   
   InitFrog(frog);
   parse_cmdline(argc,argv, frog);
   if (frog->daemon) daemonize();

   if (OpenPort(frog)<0) {
     printf("can't open serial port: %s\n",frog->device);
     exit(0);
   }
     
   if (ConfigurePort(frog)<0) {
     printf("can't configure serial port: %s\n",frog->device);
     exit(0);
   };   
   while (frog->nbretry != nberror) {
      ResetFrog(frog);
      error=FALSE;
      if (GetIdentification(frog) < 0) {
         error=TRUE;
         if (frog->nbretry) nberror++;
      }
      if ((frog->verbose) && !error) PrintIdentification(frog);
      while (error==FALSE) {
         sleep(frog->period);
         if (CalcValue(frog)<0) {
            error=TRUE;
            if (frog->nbretry) nberror++;
         }
         if ((frog->log) && !error) LogLastResult(frog);
         if ((frog->verbose) && !error) PrintLastResult(frog);
      }
   }

   ClosePort(frog);   
   if (close(frog->fd)== -1) perror("close");   
   return(1);
}




