/***********************************************************************
frogd, daemon pour l'utilisation des capteurs meteo de http://www.froggyhome.com

Copyright (C) 2001 (Regis Barbier)
Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon 
les termes de la Licence Publique Gnrale GNU publie par la Free Software 
Foundation (version 2 ou bien toute autre version ultrieure choisie par vous).

Ce programme est distribu car potentiellement utile, mais SANS AUCUNE 
GARANTIE, ni explicite ni implicite, y compris les garanties de 
commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la 
Licence Publique Gnrale GNU pour plus de dtails.

Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme 
temps que ce programme. Si ce n'est pas le cas, crivez  la Free Software 
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
*************************************************************************/

#include "frogd.h"
#include "frogd_calc.h"
#include "frogd_comm.h"


//---------------------------------------------------------------
// process the data of the presure and temperature sensors
//---------------------------------------------------------------
int CalcPresTemp(Frog* frog, int sensornumber, unsigned char* reponse) {
   unsigned  char tempo[FROG_MAX_FRAME_SIZE];
   unsigned short int Word1, Word2, Word3, Word4;
   unsigned short int C1, C2, C3, C4, C5, C6;
   unsigned short int D1, D2;
   float fUTI,fDT,fTemp;
   float fOFF,fSENS,fX,fPress;
   int w;
   
  Word1=Word2=Word3=Word4=0;
  C1=C2=C3=C4=C5=C6=0;
  
   Decode(reponse, tempo, frog->sensor[sensornumber].phymesurementsize*2+2);
   
   Word1=frog->sensor[sensornumber].data[1];
   Word1<<=8;
   Word1|=frog->sensor[sensornumber].data[2];
   Word2=frog->sensor[sensornumber].data[3];
   Word2<<=8;
   Word2|=frog->sensor[sensornumber].data[4];
   Word3=frog->sensor[sensornumber].data[5];
   Word3<<=8;
   Word3|=frog->sensor[sensornumber].data[6];
   Word4=frog->sensor[sensornumber].data[7];
   Word4<<=8;
   Word4|=frog->sensor[sensornumber].data[8];

   C1=Word1 & 0xFFFE;
   C1>>=1;
   
   C2=Word3 & 0x003F;
   C2<<=6;
   w=Word4 & 0x003F;
   C2|=w;
  
   C3=Word4 & 0xFFC0;
   C3>>=6;
   
   C4=Word3 & 0xFFC0;
   C4>>=6;

   C5=Word1 & 0x0001;
   C5<<=10;
   w=Word2 & 0xFFC0;
   w>>=6;
   C5|=w;

   C6=Word2 & 0x003F;
 
   D1=tempo[1];
   D1<<=8;
   D1|=tempo[2];

   D2=tempo[3];
   D2<<=8;
   D2|=tempo[4];
  
   if (frog->debug) {
      printf("word1 : %.4X\n",Word1);
      printf("word2 : %.4X\n",Word2);
      printf("word3 : %.4X\n",Word3);
      printf("word4 : %.4X\n",Word4);
      printf("word1 : %.2X\n",C1);
      printf("word1 : %.2X\n",C2);
      printf("word1 : %.2X\n",C3);
      printf("word1 : %.2X\n",C4);
      printf("word1 : %.2X\n",C5);
      printf("word1 : %.2X\n",C6);
      printf("D1 : %.2X\n",D1);
      printf("D2 : %.2X\n",D2);
  }
  
/*   if (D2==0) {
      bErr=TRUE;
	   return (float)0.0;
	}*/
   
   fUTI=8*(float)C5+20224;
   if ((float)D2>=fUTI) {
      fDT=D2-fUTI;
      fTemp=(200+fDT*(C6+50)/1024)/10;
   } else {
      fDT=(D2-fUTI)-( ((D2-fUTI)/128)*((D2-fUTI)/128) )/4;
	   fTemp=(200+fDT*(C6+50)/1024+fDT/256)/10;
   }

/*   if ((D1==0) && (D2==0)) {
      bErr=TRUE;
      return (float)0.0;
	}*/

   fUTI=8*(float)C5+20224;

   fOFF=C2*4+((C4-512)*fDT)/4096;
   fSENS=C1+(C3*fDT)/1024+24576;
   fX=(fSENS*(D1-7168))/16384-fOFF;
   fPress=fX*10/32+2500;
   fPress=fPress/10;
   fPress=fPress*(float)pow(10,(float) frog->altitude/19434);
   
   frog->lasttemp = fTemp;
   frog->lastpress = fPress;   

  return 1;
}

//---------------------------------------------------------------
// process the data of the humidity sensor
//---------------------------------------------------------------
int CalcHr(Frog* frog, int sensornumber, unsigned char* reponse) {
   unsigned  char tempo[FROG_MAX_FRAME_SIZE];
   unsigned short int NbBits;
   unsigned int  Rs, Rp;
   unsigned short int Mesure;
   float fImped, fHum;
   float fa,fb,fc,fd,fT2,fT3;
   float fA1,fA2,fA3,fA4,fB1,fB2,fB3,fB4,fC1,fC2,fC3,fC4,fD1,fD2,fD3,fD4;   
  
   float fTemp;

   if (frog->debug) printf("we are in CalcHr and temperature is %f\n",frog->lasttemp);
   fTemp =  frog->lasttemp;
   
   Decode(reponse, tempo, frog->sensor[sensornumber].phymesurementsize*2+2);

   NbBits=frog->sensor[sensornumber].data[1];

   Rs=Rp=0;
   Rs=frog->sensor[sensornumber].data[2];
   Rs<<=8;
   Rs|=frog->sensor[sensornumber].data[3];
   Rs<<=8;
   Rs|=frog->sensor[sensornumber].data[4];

   Rp=frog->sensor[sensornumber].data[5];
   Rp<<=8;
   Rp|=frog->sensor[sensornumber].data[6];
   Rp<<=8;
   Rp|=frog->sensor[sensornumber].data[7];
   if (frog->debug) {
      printf("Rs : %.3X\n",Rs);
      printf("Rp : %.3X\n",Rp);
   }

/*
   if (pData[0]==0xFF)
     { bErr=TRUE;
	   return (float)0.0;
	 }
  else bErr=FALSE;
*/
  
   Mesure=tempo[1];
   Mesure<<=8;
   Mesure|=tempo[2];

   if ((Mesure==0) || (Rp==0)) return (float)100.0;
   if (Mesure>=507) return (float)0.0;

   fImped=((float)Rs/1000)/((float)(pow(2, NbBits)/Mesure)-1-((float)Rs/Rp));

   fa=(float)1.0;
   fb=(float)log10(fImped);
   fc=(float)pow(fb,2);
   fd=(float)pow(fb,3);
   fT2=(float)pow(fTemp,2);
   fT3=(float)pow(fTemp,3);

   fA1=1.154564e2f;   fA2=-2.557588e1f;  fA3=4.595314e-1f;  fA4=2.008904e-1f;
   fB1=-3.066346e-1f; fB2=-8.884106e-1f; fB3=3.460639e-1f;  fB4=-3.408861e-2f;
   fC1=-2.558339e-2f; fC2=3.352567e-2f;  fC3=-1.106885e-2f; fC4=9.045758e-4f;
   fD1=3.386586e-4f;  fD2=-3.616346e-4f; fD3=9.868339e-5f;  fD4=-2.454383e-6f;

   fHum=fa*fA1+fb*fA2+fc*fA3+fd*fA4;
   fHum+=fTemp*fa*fB1+fTemp*fb*fB2+fTemp*fc*fB3+fTemp*fd*fB4;
   fHum+=fT2*fa*fC1+fT2*fb*fC2+fT2*fc*fC3+fT2*fd*fC4;
   fHum+=fT3*fa*fD1+fT3*fb*fD2+fT3*fc*fD3+fT3*fd*fD4;

   if (fHum>(float)100.0) fHum=(float)100.0;
   if (fHum<(float)0.0) fHum=(float)0.0;
  
   frog->lasthum = fHum;   
     
   return 1;
}

//---------------------------------------------------------------
// process the frame and return the values for presure, temperature and humidity
//---------------------------------------------------------------
int CalcValue(Frog* frog) {
   unsigned char tab[255];
   unsigned char* reponse;
   int nbread; 
   char* command="01";  // command 01 => ask le last values
   int i;

   if (frog->debug) printf("We are in CalcValue\n");
   
   reponse = tab;
   if ( (nbread = AskFrog(command, frog, reponse)) < 0) {
      printf("Askfrog not successfull \n");
      return -1;
   }
 
   reponse+=1;      // strip leading G 
 
   if (frog->debug) {
      for (i=0; i<nbread/2; i++) {
         printf("offset %d - values %c%c \n",i*2,reponse[i*2], reponse[i*2+1]); 
      }
      printf("frame size : %d octet\n", nbread );
   }

   reponse+=2;     // strip leading command byte

   for (i=0; i < frog->nbsensor; i++) {
      switch (frog->sensor[i].identification) {
         case SENSOR_PRES_TEMP:
            CalcPresTemp(frog, i, reponse);
            break;
         
         case SENSOR_HR:
            CalcHr(frog, i, reponse);
            break;         
      }
      reponse += frog->sensor[i].phymesurementsize*2 + 2;  // go to the next sensor
   } 
   return 1;
}
